/* ============================================================
 *
 * This file is a part of digiKam project
 * http://www.digikam.org
 *
 * Date        : 2003-10-14
 * Description : digiKam KIO thumbnails generator interface
 *
 * Copyright (C) 2003-2005 by Renchi Raju <renchi@pooh.tam.uiuc.edu>
 * Copyright (C) 2006-2007 by Gilles Caulier <caulier dot gilles at gmail dot com>
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * ============================================================ */

#ifndef THUMBNAILJOB_H
#define THUMBNAILJOB_H

// Qt includes.

#include <QByteArray>
#include <QPixmap>

// KDE includes.

#include <kio/job.h>
#include <kurl.h>
#include <kfileitem.h>

// Local includes

#include "digikam_export.h"

class QPixmap;
class QImage;

namespace Digikam
{

class ThumbnailJobPriv;

class DIGIKAM_EXPORT ThumbnailJob : public KIO::Job
{
    Q_OBJECT

public:

    ThumbnailJob(const KUrl& url, int size,
                 bool highlight=true, bool exifRotate=false);
    ThumbnailJob(const KUrl::List& urlList, int size,
                 bool highlight=true, bool exifRotate=false);
    ~ThumbnailJob();

    void addItem(const  KUrl& url);
    void addItems(const KUrl::List& urlList);

    bool setNextItemToLoad(const KUrl& url);
    void removeItem(const KUrl& url);

signals:

    void signalThumbnail(const KUrl& url, const QPixmap& pix);
    void signalCompleted();
    void signalFailed(const KUrl& url);

private:

    void processNext();
    void emitThumbnail(QImage& thumb);
    void createShmSeg();

protected slots:

    void slotResult(KJob *job);
    void slotThumbData(KIO::Job *job, const QByteArray &data);
    void gotKDEPreview(const KFileItem &, const QPixmap &);
    void failedKDEPreview(const KFileItem &);

private:

    ThumbnailJobPriv *d;
};

}  // namespace Digikam

#endif /* THUMBNAILJOB_H */
